# Functions for managing the administrative web server
#
# $Id: Admserv.pm,v 1.22.2.1 1999/01/11 22:15:21 brion Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Admserv;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(admserv_get_access admserv_set_access admserv_set_redirect
	     admserv_set_user_cgis admserv_get_user_cgis);
}
1;
__END__

sub admserv_set_access
# admserv_set_access: set the access rights to a directory
# Arguments: path, mode, type, and name
#   mode may be "any", "admin", or "valid":
#    any -- open access
#    admin -- require user admin
#    site-admin -- require a site administrator
#    valid -- require valid-user
#    require -- require the given user
#    none -- no access restrictions (remove the file)
#   The type and name arguments is optional, unless the require mode is used
#   The admin user or group are always allowed access
# Return value: status string of the usual sort
# Side effects: creates the htaccess files
# Example: Cobalt::Admserv::admserv_set_access("/home/httpd/html/joe","require","user","joe");
{
    my ($dir,$mode,$type,$user) = @_;

    return "4999 admserv_invalid_dir" 
      if (($dir !~ /^$Htusers_dir/) && ($dir !~ /^$Htgroup_dir/) && ($dir !~ /^$Top_dir/o));
    if ($mode eq "none")
    {
	if (-e "$dir/$Access_file")
	{
	    unlink("$dir/$Access_file") ||
		return "4999 admserv_cant_remove_accessfile";
	}
	return "2999 admserv_ok_opened";
    }
    open(HTA,">$dir/$Access_file") ||
	return "4999 admserv_cant_write_accessfile";
    # No explicit <Limit>, because we want to restrict all methods
    print HTA "# Access file for $dir ($mode $type $user)\n";
    print HTA "order allow,deny\n";
    print HTA "allow from all\n";
    if    ($mode eq "valid")   { print HTA "require valid-user\n";   }
    elsif ($mode eq "admin")   { print HTA "require user $Adminuser\n";   }
    elsif ($mode eq "require") { print HTA "require $type $user $Adminuser\n";}
    elsif ($mode eq "site-admin") {
      print HTA "require group $MainGroup $Sites_AdminGroup $Adminuser\n";
    }
    print HTA "Authname $Usr_Authname\nAuthtype $Authtype\n";
    print HTA "AuthUserFile $Htpasswd\n";
    print HTA "AuthGroupFile $Htgroup\n";
    close HTA;

    # web server needs to read the file
    chmod 0644, "$dir/$Access_file";

    return "2999 admserv_ok_set_rights";
}

sub admserv_get_access
# admserv_get_access: get the access rights for a directory
# Arguments: a directory name
# Return value: a mode, as above
# Example: $protected = Cobalt::Admserv::admserv_get_access("/home/admserv/.cobalt/place");
{
    my ($dir) = @_;
    return "none" if (! -e "$dir/$Access_file");
    open(HTA,"$dir/$Access_file") ||
	return "4999 admserv_cant_read_accessfile";
    while (<HTA>)
    {
	next unless (/^require/o);
	if (/^require valid/o)
	{
	    close HTA;
	    return "valid";
	}
	elsif (/^require (\w+) (.+) $Adminuser/o)
	{
	    close HTA;
	    return "require $1 $2";
	}
    }
    close HTA;
    return "any";
}

sub admserv_set_redirect
# Set the host and port to which the admin server gets redirected
# Arguments: name of the server host, port on which the admin server runs
# Side effects: modifies srm.conf, restarts httpd
# Return value: usual
# Example: Cobalt::Admserv::admserv_set_redirect("209.19.61.16",81);
{
  # This function is no longer used--we use mod_rewrite instead.
  return;

    my ($host,$port) = @_;
    my ($donc,$dond,$done,$donf,$dong,$donp,$dons);
    return "1999 admserv_unsupported_no_httpd"
      unless ($Features{'adm'});
    return "4999 admserv_cant_srm_locked"
	if (-e "$Lockdir/srm.conf");
    open(HTCTMP,">$Lockdir/srm.conf")
	|| return "4999 admserv_cant_lock_srm";
    open(HTCONF,"$Htconf/srm.conf") 
	|| return "4999 admserv_cant_read_srm";
    while (<HTCONF>)
    {
	if (/^Redirect \/.cobalt/o)
	{
	    # /.cobalt/ redirects web page accesses from the main server
	    # to the admin server
	    print HTCTMP "Redirect /.cobalt http://$host:$port/.cobalt\n";
	    $done = 1;
	}
	elsif (/^Redirect \/cgi-bin\/.cobalt/o)
	{
	    # /cgi-bin/.cobalt/ redirects CGI accessses to the admin server
	    print HTCTMP "Redirect /cgi-bin/.cobalt http://$host:$port/cgi-bin/.cobalt\n";
	    $donf = 1;
	}
	elsif ($Features{'personalRedirect'} && (/^Redirect \/personal/o))
	{
	    # /personal/ redirects web page accesses from the main server
	    # to the personal profile pages
	    print HTCTMP "Redirect /personal http://$host:$port/.cobalt/personal\n";
	    $donp = 1;
	}
	elsif ($Features{'adminRedirect'} && (/^Redirect \/admin/o))
	{
	    # /admin/ redirects web page accesses from the main server
	    # to the administrator site
	    print HTCTMP "Redirect /admin http://$host:$port/.cobalt/sysManage\n";
	    $donc = 1;
	}
	elsif ($Features{'siteAdminRedirect'} && (/^Redirect \/siteadmin/o))
	{
	    # /siteadmin/ redirects web page accesses from the main server
	    # to the site management site
	    print HTCTMP "Redirect /siteadmin http://$host:$port/.cobalt/siteManage/$MainGroup\n";
	    $dons = 1;
	}
	elsif ($Features{'cobaltRedirect'} && (/^Alias \/cobalt\//o))
	{
	    # Just record the fact that the alias is already set
	    print HTCTMP;
	    $dond = 1;
	}
	# not useful for new discussion group
#	elsif ($Features{'discuss'} && (/^Alias \/discuss/o))
#	{
	    # Just record the fact that the alias is already set
#	    print HTCTMP;
#	    $dong = 1;
#	}
	else
	{
	    print HTCTMP;
	}
    }

    # This part adds aliases and redirects if they weren't already there;
    # the above fixes existing lines in the config file.
    if ($Features{'adminRedirect'})
    {
      print HTCTMP "Redirect /admin http://$host:$port/.cobalt/sysManage\n"
	unless ($donc);
    }
    if ($Features{'siteAdminRedirect'})
    {
      print HTCTMP "Redirect /siteadmin http://$host:$port/.cobalt/siteManage/$MainGroup\n"
	unless ($dons);
    }
    if ($Features{'personalRedirect'})
    {
      print HTCTMP "Redirect /personal http://$host:$port/.cobalt/personal\n"
	unless ($donp);
    }
    if ($Features{'cobaltRedirect'})
    {
      print HTCTMP "Alias /cobalt/ $Cobalt_dir/\n" unless ($dond);
    }
    # not useful for new discussion group
#    if ($Features{'discuss'})
#    {
#      print HTCTMP "Alias /discuss/ $Discuss_html_dir/\n" unless ($dong);
#    }
    print HTCTMP "Redirect /.cobalt http://$host:$port/.cobalt\n"
	unless ($done);
    print HTCTMP "Redirect /cgi-bin/.cobalt http://$host:$port/cgi-bin/.cobalt\n" 
	unless ($donf);
    close HTCONF;
    close HTCTMP;
    rename("$Lockdir/srm.conf","$Htconf/srm.conf") 
	|| return "4999 admserv_cant_write_srm";
    Cobalt::Admserv::restart_httpd();
    return "2999 admserv_ok_redirect_updated";
}

sub admserv_set_user_cgis
# Turn CGI access for users on or off
# Arguments: 1 for on, 0 for off
# Side effects: creates the file which tells cgiwrap wether or not
#   to allow user CGIs.  cgiwrap just looks to see if the file
#   exists, and if it does it permits users to run CGIs in their
#   own area.
# Return value: the usual status string
# Example: Cobalt::Admserv::admserv_set_user_cgis(0);
{
    my ($turnon) = @_;

    return "1999 admserv_unsupported_cgis"
      unless ($Features{'www'});

    if ($turnon && -e $Allow_cgis_file)
    {
	return "2999 admserv_ok_cgis_already_on";
    }
    elsif ($turnon)
    {
	open(ALLOW,">$Allow_cgis_file")
	    || return "4999 admserv_cant_cgis_on";
	close ALLOW;
	return "2999 admserv_ok_cgis_on";
    }
    elsif (-e $Allow_cgis_file)
    {
	unlink($Allow_cgis_file)
	    || return "4999 admserv_cant_cgis_off";
	return "2999 admserv_ok_cgis_off";

    }
    else
    {
	return "2999 admserv_ok_cgis_already_off";
    }
}

sub admserv_get_user_cgis
# Report on wether user CGIs are allowed
# Arguments: none
# Return value: 1 if yes, 0 if no
# Side effects: none
# Example: $cgis_are_on = Cobalt::Admserv::admserv_get_user_cgis();
{
    (-e $Allow_cgis_file) ? (return 1) : (return 0);
}

sub restart_httpd
# Restart the web server
# Arguments: none
# Side effects: sends a HUP to the server
{
    my ($pid,$l);
    open(PID,$Htpid) 
	|| return "4999 admserv_cant_read_wwwpid";
    $l = <PID>;
    if ($l =~ /^(\d+)$/o)
    {
	$pid = $1;
    }
    close PID;
    kill('HUP',$pid) if ($pid);

    return "2999 admserv_ok_www_restarted";
}

1;
