# Vhost.pm: Functions for virtual hosting
#
# $Id: Vhost.pm,v 1.65.2.1 1999/01/08 19:21:16 gsri Exp $
#
# Written by
#   Brion Moss (brion@cobaltnet.com, brion@nda.com)
#   Kevin Chiu (kevin@cobaltnet.com)
#   Will DeHaan (will@cobaltnet.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

# We were being loose with naming.  Let's try to make this better.
# vhost_* functions manage IP-based virtual sites
# vname_* functions manage name-based virtual sites
# vsite_* functions manage virtual sites in general (name or IP)

package Cobalt::Vhost;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(vhost_set vhost_del vhost_list vhost_get_byaddr 
	     vname_add vname_del 
	     vhost_gen_name
	     vsite_list vsite_get vsite_get_bygroup
	     vhost_get_bygroup vhost_get
	     vsite_admin_add vsite_admin_del vsite_admin_set vsite_admin_list
	     vsite_set_cgis vsite_get_cgis vsite_getgroup
	     vsite_set_ssi vsite_get_ssi
	    );

require Cobalt::Group;
import Cobalt::Group qw(:DEFAULT);
require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
require Cobalt::List;
import Cobalt::List qw(:DEFAULT);
require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(:DEFAULT);
}

use integer;
1;
__END__

sub vhost_get
# Get information on a virtual site
# use Cobalt::Vhost::vsite_get() instead
{ 
  return Cobalt::Vhost::vsite_get(@_); 
}
sub vhost_get_bygroup
# get info on a virtual site based on group name
# use Cobalt::Vhost::vsite_get_bygroup() instead
{
  return Cobalt::Vhost::vsite_get_bygroup(@_); 
}

sub vname_add
# Adds a name-based virtual web server
# This function is specifically for name-based virtual sites
# Arguments: the IP address, server name, associated group name, admin email
# Return value: string of the form "3310 High Anxiety"
# Side effects: Adds NameVirtualHost entry to the conf file and <VirtualHost>
#    block
{
	my ($addr,$name,$group,$serveradmin) = @_;
	my($found_interface,$found_namev,$new_conf);

	# read in the current configuration to see if this virtual
	# site already exists and if the interface is there
	open(HTCNF,"$Htconf/httpd.conf") || 
	  return "4587 Cannot open web configuration";
        while(<HTCNF>)
        {
		$found_interface = 1 if (/VirtualHost $addr>/);
		$found_namev = 1 if (/NameVirtualHost $addr$/);
		$new_conf .= $_;
        }
        close(HTCNF);

	if ($found_interface) 
	{
	  $new_conf .= <<EOF
		  
# VirtualHost label name Please do not edit or remove this comment
<VirtualHost $addr>
ServerName $name
ServerAdmin $serveradmin
DocumentRoot $Htgroup_dir/$group
ErrorLog $Weblog_dir/$group-error
CustomLog $Weblog_dir/$group-access "%h %l %u %t \\"%r\\" %s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\""
Redirect /siteadmin http://$addr:$AdminServer_port/.cobalt/siteManage/$group
Redirect /personal http://$addr:$AdminServer_port/.cobalt/personal
#AddHandler cgi-wrapper .cgi
#AddHandler cgi-wrapper .pl
</VirtualHost>
EOF
	} else {
		return "4595 IP address is not in use.  Please create an IP-bound server.";
	}

	if (!$found_namev) 
	{
	  # This is a wholly new site
	  my @siteslist = Cobalt::Vhost::vsite_list();
	  return "4597 This system is limited to $Sites_Max virtual sites."
	    if (($#siteslist + 1) >= $Sites_Max); # +1 to un-zero-index
	  $new_conf =~ s/(Listen $addr:80)/NameVirtualHost $addr\n$1/;
	}

	# Now get our locks
	return "4377 Cannot reconfigure network: system busy"
	  unless(Cobalt::Network::net_lock_network());
	return Cobalt::Network::net_abort("4588 Web configuration in use, please try again")
	  if (-e "$Lockdir/httpd.conf");
	open(HTLOCK,"> $Lockdir/httpd.conf") || 
	  return Cobalt::Network::net_abort("4593 Cannot update the web server configuration file");
	print HTLOCK $new_conf;	
	close(HTLOCK);

	rename("$Lockdir/httpd.conf","$Htconf/httpd.conf") ||
	  return Cobalt::Network::net_abort("4589 Cannot update the web server configuration file",
			  "Lockdir/httpd.conf");
	Cobalt::Network::net_unlock_network();
	return "2660 Name-based virtual server added successfully"; 
}


sub vname_del
# Removes a name-based virtual web server
# This function is specifically for name-based sites and will do nothing
#   for IP-based ones
# Arguments: the IP address already in use, the name of the server, the
#    associated group name
# Return value: string of the form "3311 Humbolt lunch"
# Side effects: Might remove NameVirtualHost entry, removes <VirtualHost> 
#    block from the httpd.conf file
{
  my ($addr,$name,$group) = @_;
  my($found_interface,$found_namev,$new_conf,$nothinglost);

  open(HTCNF,"$Htconf/httpd.conf") || 
    return Cobalt::Network::net_abort("4590 Cannot open web configuration");
  while(<HTCNF>) {
	    if (/^\# VirtualHost label name /o)
	    {
		$nothinglost = $_;
		$_ = <HTCNF>;
		$nothinglost .= $_;

		if (/<VirtualHost $addr>/)
		{
		    $_ = <HTCNF>;
		    $nothinglost .= $_;
		    if (/ServerName $name\s*$/) 
  		    {
			$found_interface = 1;
	    	        # Skip past the rest of the entry
		        while (<HTCNF>)
            	        {
                            last if (/^<\/VirtualHost>/o);
            	        }
		    } else {
			$new_conf .= $nothinglost;
		    }
		} else {
		    $new_conf .= $nothinglost;
		}
	    } else {
	      $new_conf .= $_;
	    }
	  }
        close(HTCNF);

	return "4586 Virtual name-based web server $addr does not exist." 
	  if (!$found_interface);

  # now get our locks
  return "4378 Cannot reconfigure network: system busy"
    unless(Cobalt::Network::net_lock_network());
  return Cobalt::Network::net_abort("4591 Web configuration in use, please try again") 
    if (-e "$Lockdir/httpd.conf");

	open(HTLOCK,"> $Lockdir/httpd.conf") || 
	  return Cobalt::Network::net_abort("4594 Cannot update the web server configuration file");
	print HTLOCK $new_conf;	
	close(HTLOCK);

  rename("$Lockdir/httpd.conf","$Htconf/httpd.conf") ||
    return Cobalt::Network::net_abort("4585 Cannot update the web server configuration file",
		     "Lockdir/httpd.conf");
	
	if ($group)
	{
		unlink("$Weblog_dir/$group-access") if (-e "$Weblog_dir/$group-access");
		unlink("$Weblog_dir/$group-error") if (-e "$Weblog_dir/$group-error");
	}

  # unlock the interface
  Cobalt::Network::net_unlock_network();
	return "2661 Name-based virtual server deleted successfully"; 
}


sub vhost_set
# Configure or create the settings for an IP-based virtual interface,
#   setting up the web server and the network interface.
# This is specifically for IP-based virtual sites.
# Arguments: the new IP address, netmask, fully qualified host name,
#   group name, optional physical interface name (defaults to eth0)
# Side effects: edits the files in /etc/sysconfig
#   Reconfigures the network interface for the new address
#   Adds entries to the httpd.conf to listen to the new address
# Return value: status string of the form "4346 You don't get a wish
#   because you don't have your own brain."
# Note: this doesn't handle the mail configuration aspects of setting
#   up a virtual host, only the network interface and the web server.
{
    my ($addr,$mask,$name,$group,$netif) = @_;
    my $cgis_enabled;

    $netif ||= $Netif;

    # Picky, but not worth returning an error
    $name =~ tr/A-Z/a-z/;

#    $addr =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    # If we're given a weird IP and it's not via the web, wake the LCD
#    if ($addr !~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    if( !Cobalt::Netutil::netutil_test_ipaddr( $addr ) )
    {
	print STDERR "Got invalid IP address $addr\n" if ($Debug);
	return "4140 $addr is not a valid IP address";
    }
    if (Cobalt::Network::net_network_ismember($addr,"224.0.0.0","240.0.0.0")) {
      # This is an address on the multicast network
      return "4999 vsite_invalid_ismulticast";
    }

    # Check that the hostname is valid
    if ($name =~ /^([\w\-\.]+)$/o)
    {
	$name = $1;  # untaint
    }
    else
    {
	return "4138 invalid hostname $name";
    }

    # There must be an existing group to associate with this interface
    unless (getgrnam($group))
    {
	return "4142 Error: Group doesn't exist";
    }

    my $broadcast = Cobalt::Network::net_get_broadcast($addr,$mask);
    my $network = Cobalt::Network::net_get_network($addr,$mask);
    my $main_addr = (Cobalt::Network::net_get_ipaddr())[0];
    
    # Are we modifying an existing interface or adding a new one?
    my ($virtif) = (Cobalt::Vhost::vhost_get_byaddr($addr))[0];
    print STDERR "Virtual interface is ``$virtif''\n" if ($Debug);
    unless ($virtif)
    {
      # This is a new interface
      my @siteslist = Cobalt::Vhost::vsite_list();
      return "4598 This system is limited to $Sites_Max virtual sites."
	if (($#siteslist + 1) >= $Sites_Max);  # plus one to un-zero-index
      
	# Get the number for the next interface to add
	my $ifnum;
	opendir(IFDIR,$If_cfg_dir) || 
	    return "4143 Cannot read network script dir $If_cfg_dir: $!";
	my @ifcfgs = sort {$a <=> $b } (grep(/^ifcfg-$netif:\d+$/o,readdir(IFDIR)));
	closedir IFDIR;
	for ($ifnum = 0; $ifnum <= $#ifcfgs; $ifnum++)
	{
	    if ($ifcfgs[$ifnum] ne "ifcfg-$netif:$ifnum")
	    {
		$virtif = "$netif:$ifnum";
		last;
	    }
	}
	$virtif ||= "$netif:" . ($#ifcfgs + 1);
	print STDERR "Allocating new interface $virtif for $addr\n"
	  if ($Debug);
    }
    else {
      # this is an existing interface
      $cgis_enabled = Cobalt::Vhost::vsite_get_cgis($virtif);
    }

    print STDERR "New settings for $virtif: $addr/$mask $name $group\n"
	if ($Debug);
    # need the general lock NO MATTER WHAT
    return "4379 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    return Cobalt::Network::net_abort("4144 Cannot configure virtual interface: ifcfg-$virtif is locked")
      if (-e "$Lockdir/ifcfg-$virtif");
    open(O,">$Lockdir/ifcfg-$virtif")
      || return Cobalt::Network::net_abort("4145 Cannot get lock for interface $virtif: $!");

    # get all our locks at once
    if (-e "$Lockdir/httpd.conf")
    {
	return Cobalt::Network::net_abort("3140 $Htconf/httpd.conf is locked",
			 "$Lockdir/ifcfg-$virtif");
    }
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
	return Cobalt::Network::net_abort("3141 Cannot write $Lockdir/httpd.conf: $!",
			 "$Lockdir/ifcfg-$virtif");
    }

    print O "DEVICE=$virtif\n";
    print O "IPADDR=$addr\n";
    print O "NETMASK=$mask\n";
    print O "NETWORK=$network\n";
    print O "BROADCAST=$broadcast\n";
    print O "ONBOOT=yes\nALIAS=yes\nBOOTPROTO=none\n";
    close O;

    my %listenaddrs;
    my $donv = 0;
    unless (open(HTCONF,"$Htconf/httpd.conf"))
    {
	return Cobalt::Network::net_abort("3142 Cannot read $Htconf/httpd.conf: $!",
			"$Lockdir/ifcfg-$virtif",
			"$Lockdir/httpd.conf");
    }
    while (<HTCONF>)
    {
	if (/^Listen ([^:]+)\:(\d+)/o)
	{
	    $listenaddrs{$1} = $2;
	    print HTCTMP;
	}
	elsif (/^<VirtualHost $addr>/)
	{
	    # There's already an entry for us, rewrite it
	    print STDERR "Fixing virtual host entry\n" if ($Debug);
	    print HTCTMP<<END;
# VirtualHost label $virtif Please do not edit or remove this comment
<VirtualHost $addr>
ServerName $name
ServerAdmin $group\@$name
DocumentRoot $Htgroup_dir/$group
CustomLog $Weblog_dir/$group-access "%h %l %u %t \\"%r\\" %s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\""
ErrorLog $Weblog_dir/$group-error
Redirect /siteadmin http://$addr:$AdminServer_port/.cobalt/siteManage/$group
Redirect /personal http://$addr:$AdminServer_port/.cobalt/personal
END
;
            if ($cgis_enabled) {
               print "AddHandler cgi-wrapper .cgi\nAddHandler cgi-wrapper .pl\n";
            }
            else {
               print "#AddHandler cgi-wrapper .cgi\n#AddHandler cgi-wrapper .pl\n";
            }
            print "</VirtualHost>\n";
	    $donv = 1;
	    
	    # Skip past the rest of the entry
	    while (<HTCONF>)
	    {
		last if (/^<\/VirtualHost>/o);
	    }
	}
	else
	{
	    print HTCTMP;
	}
    }
    print HTCTMP "Listen ${main_addr}:80\n" unless $listenaddrs{$main_addr};
    print HTCTMP "Listen ${addr}:80\n" unless $listenaddrs{$addr};
    unless ($donv)
    {
	print STDERR "Adding new virtual host entry\n" if ($Debug);
	print HTCTMP<<END; 
# VirtualHost label $virtif Please do not edit or remove this comment
<VirtualHost $addr>
ServerName $name
ServerAdmin $group\@$name
DocumentRoot $Htgroup_dir/$group
CustomLog $Weblog_dir/$group-access "%h %l %u %t \\"%r\\" %s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\""
ErrorLog $Weblog_dir/$group-error
Redirect /siteadmin http://$addr:$AdminServer_port/.cobalt/siteManage/$group
Redirect /personal http://$addr:$AdminServer_port/.cobalt/personal
#AddHandler cgi-wrapper .cgi
#AddHandler cgi-wrapper .pl
</VirtualHost>
END
    }
    close HTCONF;
    close HTCTMP;
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("3143 Cannot update $Htconf/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }

    unless (open(LOGROT,">$Logrotate_dir/$addr-apache"))
    {
      return Cobalt::Network::net_abort("3144 Cannot create $Logrotate_dir/$addr-apache: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }
    print LOGROT <<END;
$Weblog_dir/$group-access {
   compress
   size 30M
    postrotate
        killall -USR1 httpd
    endscript
    }

$Weblog_dir/$group-error {
    compress
    size 5M
    postrotate
        killall -USR1 httpd
    endscript
    }
END

    unless (open(LOGROT,">$Logrotate_boot/$addr-apache"))
    {
      return Cobalt::Network::net_abort("3146 Cannot create $Logrotate_boot/$addr-apache: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }
    print LOGROT <<END;
$Weblog_dir/$group-access {
   compress
   size 30M
    }

$Weblog_dir/$group-error {
    compress
    size 5M
    }
END

    # unlock
    unless (rename("$Lockdir/ifcfg-$virtif","$If_cfg_dir/ifcfg-$virtif"))
    {
	return Cobalt::Network::net_abort("3145 Cannot update virtual interface config file: $!",
			 "$Lockdir/ifcfg-$virtif");	 
    }

    # Bring the interface up
    print STDERR "$Ifup $virtif\n" if ($Debug);
    system("$Ifup $virtif >>$Sauce_log 2>&1\n");

    # Restart the web server, if it's up
    if (-f "$Subsys/httpd")
    {
	system("$Htscript stop >>$Sauce_log 2>&1");
	system("$Htscript start >>$Sauce_log 2>&1");
	system("$Lcdshowscript") if (-x $Lcdshowscript);
    }

    Cobalt::Network::net_unlock_network();

    if (-r $Dns_notification)
    {
        my($subject) = Cobalt::Language::msg_get('notify_vhost_add_subject');
        $subject =~ s/\$addr/$addr/; $subject =~ s/\$main_addr/$main_addr/; $subject =~ s/\\n/\n/g;
        my($msgbody) = Cobalt::Language::msg_get('notify_vhost_add_body');
        $msgbody =~ s/\$addr/$addr/; $msgbody =~ s/\$main_addr/$main_addr/; $msgbody =~ s/\\n/\n/g;
        Cobalt::Vhost::inform_BigBrother($subject,$msgbody);
    }

    return "2140 Set up virtual interface for address $addr";
}

sub vhost_list
# Get a list of virtual interfaces (spans all physical interfaces)
# This only lists IP-based virtual sites
# Arguments: none
# Return value: a list of interfaces ("eth0:0","eth0:1","eth1:0",...)
{
    opendir(IFDIR,$If_cfg_dir) || 
	return "4148 Cannot read network script dir $If_cfg_dir: $!";
    my @ifcfgs = map { substr($_,6) }  # skip the ``ifcfg-'' part of the name
	sort(grep(/^ifcfg-eth\d+:\d+$/o,readdir(IFDIR)));
    closedir IFDIR;
    return @ifcfgs;
}

sub vhost_get_byaddr
# Get information about a virtual interface based on the IP address
# This only shows IP-based virtual sites
# Arguments: IP address
# Return value: interface, netmask, hostname, group
#   Returns 0 if we don't serve this (or if it isn't a virtual interface)
# Example: ($virtif,$mask,$host,$group) = Cobalt::Vhost::vhost_get_byaddr("192.168.41.5");
{
    my ($addr) = @_;
    my @virtifs = Cobalt::Vhost::vhost_list();
    my $virtif;
    foreach $virtif (@virtifs)
    {
	my ($ipaddr,$mask,$host,$group) = Cobalt::Vhost::vsite_get($virtif);
	if ($ipaddr eq $addr)
	{
	    return($virtif,$mask,$host,$group);
	}
    }
    return 0;
}

sub vhost_del
# Remove an IP-based virtual interface
# This only applies to IP-based virtual sites
# Arguments: an interface name, the associated group name
# Side effects: removes the ifcfg-eth0:xx file, the /etc/logrotate.d file,
#   cleans httpd.conf
{
    my ($virtif,$group) = @_;
    unless (open(IFCFG,"$If_cfg_dir/ifcfg-$virtif"))
    {
	return "4154 Cannot read $If_cfg_dir/ifcfg-$virtif: $!";
    }
    my $addr;
    while (<IFCFG>)
    {
	if (/^IPADDR=(.+)/o)
	{
	    $addr = $1;
	    last;
	}
    }
    close IFCFG;
    # get the main network lock file (and the rest of our locks)
    return "4386 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    return "4150 Cannot remove interface: ifcfg-$virtif is locked"
	if (-e "$Lockdir/ifcfg-$virtif");
    open(O,">$Lockdir/ifcfg-$virtif")
	|| return "4151 Cannot lock interface config ifcfg-$virtif: $!";
    close O;
    if (-e "$Lockdir/httpd.conf")
    {
      return Cobalt::Network::net_abort("4149 $Htconf/httpd.conf is locked",
		       "$Lockdir/ifcfg-$virtif");
    }
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4152 Cannot write $Lockdir/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif");
    }
    unless (open(HTCONF,"$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4153 Cannot read $Htconf/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }
    while (<HTCONF>)
    {
	if (/^Listen $addr:80/o)
	{
	    next;
	}
	elsif (/^<VirtualHost $addr>/o)
	{
	    while (<HTCONF>)
	    {
		last if (/^<\/VirtualHost>/o);
	    }
	}
	elsif (/^\# VirtualHost label $virtif /) 
	{
	    next;
	}
	else
	{
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("3149 Cannot update $Htconf/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }

    unlink("$Logrotate_dir/$addr-apache");
    unlink("$Logrotate_boot/$addr-apache");
    system("$Ifdown $virtif >>$Sauce_log 2>&1");
    unlink("$If_cfg_dir/ifcfg-$virtif");
    unlink("$Lockdir/ifcfg-$virtif");

    if ($group)
    {
      # conditional for backwards compatibility Cobalt::Vhost::vhost_del(nic)
      unlink("$Weblog_dir/$group-access") 
	if (-e "$Weblog_dir/$group-access");
      unlink("$Weblog_dir/$group-error") 
	if (-e "$Weblog_dir/$group-error");
    }

    Cobalt::Network::net_unlock_network();

    if (-r $Dns_notification)
    {
	my $main_addr = (Cobalt::Network::net_get_ipaddr())[0];
        
	my($subject) = Cobalt::Language::msg_get('notify_vhost_del_subject');
        $subject =~ s/\$addr/$addr/; $subject =~ s/\$main_addr/$main_addr/; $subject =~ s/\\n/\n/g;
        
        my($msgbody) = Cobalt::Language::msg_get('notify_vhost_del_body');
        $msgbody =~ s/\$addr/$addr/; $msgbody =~ s/\$main_addr/$main_addr/; $msgbody =~ s/\\n/\n/g;
       
	Cobalt::Vhost::inform_BigBrother($subject,$msgbody);
    }

    return "2142 Virtual interface deleted successfully";
}

sub vhost_gen_name
# Generate a back-end name that is associated with a hostname/vhost
# Arguments: a host name
# Return value: an appropriate name, at least two less characters than 
#   the maximum defined name length
{
        my $groupName=$_[0];
	my $maxlength = ($MaxNameLen);

        # Disallow generic www1, www2 naming
        $groupName=~s/^www\d*\.(\w+)/$1/;
        $groupName=substr( $groupName, 0, $maxlength );

	# no trailing dot
	$groupName=~s/\.+$//o;

        # check for uniqueness
        my $nameIndex=1;
        my $g=$groupName;
        while( Cobalt::Group::group_exist( $g ) ) {
            if( length( $groupName.$nameIndex )>$maxlength ) {
                $g=substr( $groupName, 0, length( $groupName )-length( $nameIndex ) ).$nameIndex;
            }
            else { $g=$groupName.$nameIndex; }

            $nameIndex++;
        }

	return($g);
}

sub vsite_get
# Get the information about an virtual site
# This function works for either IP-based or name-based sites
# Arguments: Interface name (eth0:4 or whatever) or site name
# Return value: IP address, netmask, fully qualified host name, group,
#     webmaster email
#   Returns 0 if the given site doesn't exist
# Side effects: reads the ifcfg-NAME and httpd.conf files
# Example: ($addr,$mask,$host,$group,$admin) = Cobalt::Vhost::vsite_get("eth1:3");
{
    my ($virtif) = @_;
    my ($addr,$mask,$host,$group,$admin_email);

    if ($virtif eq $Netif) # Special case for eth0 compatibility
    {
	my $nul;
	($addr,$mask,$nul) = Cobalt::Network::net_get_ipaddr();
	$host = Cobalt::Network::net_get_hostname();
        my $domain = (net_get_dnsclient())[0];
        $host .= '.' . $domain if ($domain);
	$group = $MainGroup;
	$admin_email = ''; # Not yet supported

	return($addr,$mask,$host,$group,$admin_email); 
    }

    my $inentry = 0;
    open(HTCONF,"$Htconf/httpd.conf") 
	|| return "4147 Cannot read $Htconf/httpd.conf: $!";
    while (<HTCONF>)
    {
        if (/^\# VirtualHost label\s*(name)?\s*([a-z\:\d\.]+) /o)
	{
	  my $site = $2;
	  $inentry = 1 if ($site eq $virtif);
	}
	elsif (($inentry) && (/^<VirtualHost ([\d\.]+)>/o))
	{
	    $addr = $1;
	    print STDERR "Found address for $addr\n" if ($Debug);
	}
	elsif (($inentry) && (/^ServerAdmin ([\w\@\-\.]+)/o))
	{
	    $admin_email = $1;
	    print STDERR "Found admin email is $admin_email\n" if ($Debug);
	}
	elsif (($inentry) && (/^ServerName (.+)/o))
	{
	    $host = $1;
	    print STDERR "Found host entry for $addr: $host\n" if ($Debug);
	}
	elsif (($inentry) && (/^DocumentRoot $Htgroup_dir\/(.+)/o))
	{
	    $group = $1;
	    close HTCONF;
	    last;
	}
	elsif (/^<\/VirtualHost/o)
	{
	    $inentry = 0;  # end of entry
	}
    }

    return 0 unless ($addr);

    if (-e "$If_cfg_dir/ifcfg-$virtif") {
      # IP-based
      print STDERR "Getting information about interface $virtif\n" if ($Debug);
      open(IFCFG,"$If_cfg_dir/ifcfg-$virtif")
	|| return "4146 Cannot read interface config ifcfg-$virtif: $!";
      while (<IFCFG>) {
	if (/^IPADDR=(.+)/o)     { $addr = $1; }
	elsif (/^NETMASK=(.+)/o) { $mask = $1; }
      }
      close IFCFG;
    }
    else {
      # name-based
      $mask = "N/A";   # mask isn't important
    }

    return ($addr,$mask,$host,$group,$admin_email);
}

sub vsite_get_bygroup
# Get the information about a virtual interface by group name
# Works for either IP-based or name-based sites
# Arguments: group name
# Return value: interface or name; returns 0 if this group isn't associated
#   with any interface
# Side effects: calls Cobalt::Vhost::vsite_list() to read the httpd.conf file
# Example: $virtif = Cobalt::Vhost::vsite_get_bygroup("ratdogs");
{
    my ($group) = @_;
    return $Netif if ($group eq $MainGroup);
    my ($vhost);
    foreach $vhost (Cobalt::Vhost::vsite_list()) {
      if ($group eq $$vhost[3]) {
	return $$vhost[0];
      }
    }
    return 0;
}

sub vsite_list
# Finds all virtual web interfaces, both ip and name-based.
# Arguments: none
# Return value: nested array of:
#    (interface||'name',ip,hostname,group,webmaster email)
#    sorted by hostname
# Example: @vhosts = Cobalt::Vhost::vsite_list();
#          foreach $vhost (@vhosts) { &use( @$vhost ); }
#    In this case &use is passed the simple array described above
# Parses httpd.conf
# Note that IP-based virtual sites are referred to by interface name
#  (e.g., eth0:9) while name-based are referred to by site name (e.g.,
#  "www.cobaltsys.com")
{
        my(@vnames,$nic,$ip,$name,$adminemail,$group);
        open(HTCNF,"$Htconf/httpd.conf") || 
	  return "4592 Cannot open web configuration: $!";
	while(<HTCNF>) 
	{ 
            if (/^\# VirtualHost label ([a-z\:\d\.]+) /o)
	    {
		$nic = $1;
		$_ = <HTCNF>;
		$ip = $1 if (/^<VirtualHost ([\d\.]+)>/o);
		$_ = <HTCNF>;
		$name = $1 if (/^ServerName ([a-z\d\.\-]+)/o);
		$_ = <HTCNF>;
		$adminemail = $1 if (/^ServerAdmin ([\w\@\-\.]+)/o);
		$_ = <HTCNF>;
		$group = $1 if (/^DocumentRoot $Htgroup_dir\/([\w\.\-]+)/o);
		push( @vnames, ([$nic,$ip,$name,$group,$adminemail]) ); 
		$nic = $ip = $name = $adminemail = $group = '';
	    }
	}
	close(HTCNF);

	# sort by hostname
	@vnames=sort { ${ $a }[ 2 ] cmp ${ $b }[ 2 ] } @vnames;

	return (@vnames);
}

sub vsite_admin_list
# Lists the site admins for the given site
# Arguments: virtual interface (eth0:4) or site name (if name-based)
#   With no arguments, lists *all* site administrators
# Return value: the list of admins for that site
# Side effects: none
# Example: (@admins) = Cobalt::Vhost::vsite_admin_list("eth1:12");
{
  my ($virtsite) = @_;
  if ($virtsite) {
    # be specific
    my ($addr,$mask,$host,$group,$admin) = Cobalt::Vhost::vsite_get($virtsite);
    return Cobalt::Group::group_show($group);
  }
  else {
    # list everyone
    my @admins;
    foreach $virtsite (Cobalt::Vhost::vsite_list()) {
      push(@admins,Cobalt::Group::group_show($$virtsite[3]));
    }
    @admins = sort(@admins);

    # sanity check...am I being too paranoid?
    my @site_admins = Cobalt::Group::group_show($Sites_AdminGroup);
    if ($#site_admins ne $#admins) {
      print STDERR "Warning: site admin lists are out of sync.  Fixing...\n";
      Cobalt::Group::group_set_membership($Sites_AdminGroup,@admins);
    }

    return @admins;
  }
}

sub vsite_admin_set
# Set the list of site administrators for a virtual site
# Arguments: virtual site name or interface, list of users
# Side effects: sets the membership of the group for that site
#   to the given list, adds the site admins to the global
#   site admins group
# Return value: the usual status string of the form "3667 You are in
#   a dark place.  You are likely to be eaten by a grue."
{
  my ($vsite,@admins) = @_;
  my ($addr,$mask,$host,$group,$admin) = Cobalt::Vhost::vsite_get($vsite);
  my @current_admins = Cobalt::Group::group_show($group);
  Cobalt::Group::group_del_user($Sites_AdminGroup,@current_admins);

  Cobalt::Vhost::vsite_get_siteadmin_ftphelp(@current_admins);
  
  my $ret = Cobalt::Group::group_add_user($Sites_AdminGroup,@admins);
  return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

  Cobalt::Vhost::vsite_set_siteadmin_ftphelp($group,@admins);

  return Cobalt::Group::group_set_user($group,@admins);
}

sub vsite_admin_add
# Add a site administrator to a virtual site
# Arguments: virtual site name or interface, list of users to add
# Side effects: adds the user(s) to the group for that site
#   and to the global site admins group
# Return value: the usual status string of the form "4210 Not a Jedi yet"
{
  my ($vsite,@admins) = @_;
  my ($addr,$mask,$host,$group,$admin) = Cobalt::Vhost::vsite_get($vsite);
  my $ret = Cobalt::Group::group_add_user($Sites_AdminGroup,@admins);
  return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group
  
  # Add assorted $FtpAdminHelp files
  Cobalt::Vhost::vsite_set_siteadmin_ftphelp($group,@admins);
  
  return Cobalt::Group::group_add_user($group,@admins);
}

sub vsite_admin_del
# Remove a site administrator from a virtual site
# Arguments: virtual site name or interface, list of users to remove
#   from the admin list
# Side effects: removes the user(s) from the group for that site
#   and the global site admins group
# Return value: the usual status string of the form "2600 Game over, insert
#   coin"
# Example: Cobalt::Vhost::vsite_admin_del("eth10:1",@doomedusers);
{
  my ($vsite,@admins) = @_;
  my ($addr,$mask,$host,$group,$admin) = Cobalt::Vhost::vsite_get($vsite);
  my $ret = Cobalt::Group::group_del_user($Sites_AdminGroup,@admins);
  return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

  # Delete assoted $FtpAdminHelp files
  Cobalt::Vhost::vsite_get_siteadmin_ftphelp(@admins);

  return Cobalt::Group::group_del_user($group,@admins);
}

sub vsite_set_ssi
# Enable or disable the use of SSIs for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form "4400 not going to happen in java"
# Side effects: modifies the virtual host entry in httpd.conf
#               or the main site in access.conf
# Example: Cobalt::Vhost::vsite_set_ssi("home",1);
{
  my ($vsite,$ssi_enabled) = @_;
  my $inentry = 0;
  my $on;

  # verify and grab our locks
  return "4387 Cannot reconfigure network: system busy"
    unless(Cobalt::Network::net_lock_network());
  if (-e "$Lockdir/httpd.conf")
  {
    return Cobalt::Network::net_abort("4165 $Htconf/httpd.conf is locked");
  }
  # special case for the primary network interface
  if (($vsite eq $Netif) || ($vsite eq $MainGroup)) {
    unless (open(HTCTMP,">$Lockdir/access.conf"))
    {
      return Cobalt::Network::net_abort("4164 Cannot write $Lockdir/access.conf: $!",
                       "$Lockdir/access.conf");
    }

    unless (open(HTCONF,"$Htconf/access.conf")) {
      return "4168 Cannot read $Htconf/access.conf: $!";
    }
    while (<HTCONF>) {
      if ($inentry == 2)
      {
        # already did our work; let's not do pattern matching we don't
        # have to do.
        print HTCTMP;
      }
      elsif (/^<Directory $Userdir\/$MainGroup>/o)
      {
        $inentry = 1;
        print HTCTMP;
        print STDERR "vsite_set_ssi found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler server-parsed /o))
      {
        # clear the existing AddHandler entries
        print STDERR "vsite_set_ssi found AddHandler .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddType\s+text\/html\s+.shtml/o))
      {
        # clear the existing AddHandler entries
        print STDERR "vsite_set_ssi found AddType .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/Directory/o))
      {
        # Put in the correct new AddHandler entries
        if ($ssi_enabled) {
          print HTCTMP "AddHandler server-parsed .shtml\n";
          print HTCTMP "AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('On_string');
        }
        else {
          print HTCTMP "#AddHandler server-parsed .shtml\n";
          print HTCTMP "#AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('Off_string');
        }
        $inentry = 2;  # end of entry flag
        print HTCTMP;
      }
      else
      {
        print HTCTMP;
      }
    }
    unless ($inentry) {
      # Haven't seen it, add it to the end
      print HTCTMP "<Directory $Userdir/$MainGroup>\n";
      print HTCTMP "Options Includes Indexes SymLinksIfOwnerMatch\n";
      print HTCTMP "AllowOverride All\n";
      if ($ssi_enabled) {
        print HTCTMP "AddHandler server-parsed .shtml\n";
        print HTCTMP "AddType    text/html     .shtml\n";
        $on = Cobalt::Language::msg_get('On_string');
      }
      else {
        print HTCTMP "#AddHandler server-parsed .shtml\n";
        print HTCTMP "#AddType    text/html     .shtml\n";
        $on = Cobalt::Language::msg_get('Off_string');
      }
      $inentry = 2;  # end of entry flag
      print HTCTMP "</Directory>\n";
    }
    close HTCONF;
    close HTCTMP;

    # move the temp file in place
    unless (rename("$Lockdir/access.conf","$Htconf/access.conf"))
    {
      return Cobalt::Network::net_abort("4170 Cannot update $Htconf/access.conf: $!",
                       "$Lockdir/access.conf");
    }
  }
  else {
    # General virtual host case
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4171 Cannot write $Lockdir/httpd.conf: $!",
                       "$Lockdir/httpd.conf");
    }

    unless (open(HTCONF,"$Htconf/httpd.conf"))  {
      return Cobalt::Network::net_abort("4163 Cannot read $Htconf/httpd.conf: $!",
                       "$Lockdir/httpd.conf");
    }

    # fix the httpd.conf file
    while (<HTCONF>) {
      if ($inentry == 2) {
        # already did our work; let's not do pattern matching we don't
        # have to do.
        print HTCTMP;
      }
      elsif (/^DocumentRoot $Userdir\/$vsite\s*$/o) # Sensitive to vsite GROUP name
      {
        $inentry = 1;
        print HTCTMP;
        print STDERR "vsite_set_ssi found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler server-parsed .shtml/o))
      {
        # clear the existing AddHandler .shtml entries
        print STDERR "vsite_set_ssi found AddHandler .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddType\s+text\/html\s+.shtml/o))
      {
        # clear the existing AddType .shtml entries
        print STDERR "vsite_set_ssi found AddType .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/VirtualHost/o))
      { 
        # Put in the correct new AddHandler entries
        if ($ssi_enabled) {
          print HTCTMP "AddHandler server-parsed .shtml\n";
          print HTCTMP "AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('On_string');
        }
        else {
          print HTCTMP "#AddHandler server-parsed .shtml\n";
          print HTCTMP "#AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('Off_string');
        }
        $inentry = 2;  # end of entry flag
        print HTCTMP;
      }
      else
      {
        print HTCTMP;
      }
    }
    close HTCONF;
    close HTCTMP;

    unless ($inentry) {
      # never found the virtual host
      return Cobalt::Network::net_abort("4167 No virtual host entry for $vsite.",
                       "$Lockdir/httpd.conf");
    }

    # move the temp file in place
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4162 Cannot update $Htconf/httpd.conf: $!",
                       "$Lockdir/httpd.conf");
    }
  }
  # Restart the web server, if it's up
  if (-f "$Subsys/httpd")
  {
    print STDERR "Restarting web server" if ($Debug);
    system("$Htscript stop >>$Sauce_log 2>&1");
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript") if (-x $Lcdshowscript);
  }

  Cobalt::Network::net_unlock_network();
  return "2999 vsite_ok_set_ssi";
}

sub vsite_get_ssi
# Get the current SSI settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vhost::vsite_get_ssi("home");
{
  my ($vsite) = @_;
  my $inentry;

  # special case for the primary network interface
  if (($vsite eq $Netif) || ($vsite eq $MainGroup)) {
    unless (open(HTCONF,"$Htconf/access.conf")) {
      return "4169 Cannot read $Htconf/access.conf: $!";
    }
    while (<HTCONF>) {
      if (/^<Directory $Userdir\/$MainGroup>/o) {
        # start of entry for the main site
        $inentry = 1;
      }
      elsif ($inentry && (/^\#AddHandler server-parsed .shtml/o)) {
        # looks like it's off
        close HTCONF;
        return 0;
      }
      elsif ($inentry && (/^AddHandler server-parsed .shtml/o)) {
        # looks like it's on
        close HTCONF;
        return 1;
      }
    }
  }
  else {
    # the general case for all other virtual sites
    unless (open(HTCONF,"$Htconf/httpd.conf")) {
      return "4166 Cannot read $Htconf/httpd.conf: $!";
    }
    while (<HTCONF>) {
      if (/^DocumentRoot $Userdir\/$vsite\s*$/o) {
        # start of entry for this site
        $inentry = 1;
      }
      elsif ($inentry && (/^\#AddHandler server-parsed .shtml/o)) {
        # looks like it's off
        close HTCONF;
        return 0;
      }
      elsif ($inentry && (/^AddHandler server-parsed .shtml/o)) {
        # looks like it's on
        close HTCONF;
        return 1;
      }
    }
  }
  return 0;
}

sub vsite_set_cgis
# Enable or disable the use of CGIs for a virtual site
# This affects the entire site
# Arguments: site name, 1 to enable/0 to disable
# Return value: status string of the form "8750 I can't hear you, 
#   there's a bannana in my ear!"
# Side effects: modifies the virtual host entry in httpd.conf.
# Example: Cobalt::Vhost::vsite_set_cgis("eth0:0",1);
{
  my ($vsite,$cgis_enabled) = @_;
  my $inentry = 0;
  my $on;

  # verify and grab our locks
  return "4387 Cannot reconfigure network: system busy"
    unless(Cobalt::Network::net_lock_network());
  if (-e "$Lockdir/httpd.conf")
  {
    return Cobalt::Network::net_abort("4165 $Htconf/httpd.conf is locked");
  }
  # special case for the primary network interface
  if (($vsite eq $Netif) || ($vsite eq $MainGroup)) {
    unless (open(HTCTMP,">$Lockdir/access.conf"))
    {
      return Cobalt::Network::net_abort("4164 Cannot write $Lockdir/access.conf: $!",
		       "$Lockdir/access.conf");       
    }

    unless (open(HTCONF,"$Htconf/access.conf")) {
      return "4168 Cannot read $Htconf/access.conf: $!";
    }
    while (<HTCONF>) {
      if ($inentry == 2)
      {
	# already did our work; let's not do pattern matching we don't
	# have to do.
	print HTCTMP;
      }
      elsif (/^<Directory $Userdir\/$MainGroup>/o)
      {
	$inentry = 1;
	print HTCTMP;
	print STDERR "vsite_set_cgis found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler cgi-wrapper /o))
      {
	# clear the existing AddHandler entries 
	print STDERR "vsite_set_cgis found AddHandler\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/Directory/o))
      {
	# Put in the correct new AddHandler entries
	if ($cgis_enabled) {
	  print HTCTMP "AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('On_string');
	}
	else {
	  print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "#AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('Off_string');
	}
	$inentry = 2;  # end of entry flag
	print HTCTMP;
      }
      else
      {
	print HTCTMP;
      }
    }
    unless ($inentry) {
      # Haven't seen it, add it to the end
      print HTCTMP "<Directory $Userdir/$MainGroup>\n";
      print HTCTMP "Options Indexes SymLinksIfOwnerMatch\n";
      print HTCTMP "AllowOverride AuthConfig Limit\n";
      if ($cgis_enabled) {
	print HTCTMP "AddHandler cgi-wrapper .cgi\n";
	print HTCTMP "AddHandler cgi-wrapper .pl\n";
	$on = Cobalt::Language::msg_get('On_string');
      }
      else {
	print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
	print HTCTMP "#AddHandler cgi-wrapper .pl\n";
	$on = Cobalt::Language::msg_get('Off_string');
      }
      $inentry = 2;  # end of entry flag
      print HTCTMP "</Directory>\n";
    }
    close HTCONF;
    close HTCTMP;

    # move the temp file in place
    unless (rename("$Lockdir/access.conf","$Htconf/access.conf"))
    {
      return Cobalt::Network::net_abort("4170 Cannot update $Htconf/access.conf: $!",
		       "$Lockdir/access.conf");       
    }
  }
  else {
    # General virtual host case
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4171 Cannot write $Lockdir/httpd.conf: $!",
		       "$Lockdir/httpd.conf");       
    }

    unless (open(HTCONF,"$Htconf/httpd.conf"))  {
      return Cobalt::Network::net_abort("4163 Cannot read $Htconf/httpd.conf: $!",
		       "$Lockdir/httpd.conf");       
    }
    
    # fix the httpd.conf file
    while (<HTCONF>) {
      if ($inentry == 2) {
	# already did our work; let's not do pattern matching we don't
	# have to do.
	print HTCTMP;
      }
      elsif (/^\# VirtualHost label\s*(name)?\s*([a-z\:\d\.]+) /o)
      {
	$inentry = 1 if ($2 eq $vsite);
	print HTCTMP;
	print STDERR "vsite_set_cgis found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler cgi-wrapper /o))
      {
	# clear the existing AddHandler entries 
	print STDERR "vsite_set_cgis found AddHandler\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/VirtualHost/o))
      {
	# Put in the correct new AddHandler entries
	if ($cgis_enabled) {
	  print HTCTMP "AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('On_string');
	}
	else {
	  print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "#AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('Off_string');
	}
	$inentry = 2;  # end of entry flag
	print HTCTMP;
      }
      else
      {
	print HTCTMP;
      }
    }
    close HTCONF;
    close HTCTMP;

    unless ($inentry) {
      # never found the virtual host
      return Cobalt::Network::net_abort("4167 No virtual host entry for $vsite.",
		       "$Lockdir/httpd.conf"); 
    }

    # move the temp file in place
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4162 Cannot update $Htconf/httpd.conf: $!",
		       "$Lockdir/httpd.conf");
    }
  }
  # Restart the web server, if it's up
  if (-f "$Subsys/httpd")
  {
    print STDERR "Restarting web server" if ($Debug);
    system("$Htscript stop >>$Sauce_log 2>&1");
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript") if (-x $Lcdshowscript);
  }

  Cobalt::Network::net_unlock_network();
  return "2144 Site CGI permissions are now $on";
}

sub vsite_get_cgis
# Get the current CGI settings for a virtual site
# Arguments: the site name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vhost::vsite_get_cgis("eth0:0");
{
  my ($vsite) = @_;
  my $inentry;

  # special case for the primary network interface
  if (($vsite eq $Netif) || ($vsite eq $MainGroup)) {
    unless (open(HTCONF,"$Htconf/access.conf")) {
      return "4169 Cannot read $Htconf/access.conf: $!";
    }
    while (<HTCONF>) {
      if (/^<Directory $Userdir\/$MainGroup>/o) {
	# start of entry for the main site
	$inentry = 1;
      }
      elsif ($inentry && (/^\#AddHandler cgi-wrapper/o)) {
	# looks like it's off
	close HTCONF;
	return 0;
      }
      elsif ($inentry && (/^AddHandler cgi-wrapper/o)) {
	# looks like it's on
	close HTCONF;
	return 1;
      }
    }
  }
  else {
    # the general case for all other virtual sites
    unless (open(HTCONF,"$Htconf/httpd.conf")) {
      return "4166 Cannot read $Htconf/httpd.conf: $!";
    }
    while (<HTCONF>) {
      if (/^\# VirtualHost label\s*(name)?\s*([a-z\:\d\.]+) /o) {
	# start of entry for this site
	$inentry = 1 if ($2 eq $vsite);
      }
      elsif ($inentry && (/^\#AddHandler cgi-wrapper/o)) {
	# looks like it's off
	close HTCONF;
	return 0;
      }
      elsif ($inentry && (/^AddHandler cgi-wrapper/o)) {
	# looks like it's on
	close HTCONF;
	return 1;
      }
    }
  }
  return 0;
}

sub vsite_getgroup
# Given a user name, figure out what site they are associated with
# Arguments: user name
# Return value: site name
# Side effects: none
{
  my ($user) = @_;
  my ($site);
  my ($login,$home) = (getpwnam($user))[0,7];
  if ($home =~ /^$Userdir\/([\w\-\.]+)/o) {
    $site = $1;
  }
  my @vsite_users = Cobalt::List::list_get_mem("${site}-users");
  if (grep($site,@vsite_users)) {
    return $site;
  } else {
    return "";
  }
}

sub inform_BigBrother
{
# If $Dns_notification is readable and defined with an email address,
# send snitch-mail of any activity that requires an informed 3rd party
#
# Feature added by SimpleNet request 08/98 /wdh
#
# Arguments: subject string, text body string
        my($subject,$body) = @_;
        my($notifaddr);

        if (-r $Dns_notification)
        {
                open(NOTIF,$Dns_notification);
                chomp($notifaddr = <NOTIF>);
                close(NOTIF);

                # recipient sanity check
                return 0 if ($notifaddr !~ /^\w+\@\w+/);

                open(MALR,"| $Sendmail -oi -t") || return 0;
                print MALR <<EOF;
To: $notifaddr
From: admin
Subject: $subject

$body

EOF
                close(MALR);
        }

        # Intended to run without $Dns_notification set as well
        return 1;
}

sub vsite_set_siteadmin_ftphelp
{
  my($group,@admins) = @_;

  # Add site admin ftp instructions
  if ($Features{'proftpd'})
  {
    if (-r $FtpAdminHelpSrc)
    {
      my($ftphelpfile);
      open(FTPH,"$FtpAdminHelpSrc");
      while(<FTPH>) { $ftphelpfile .= $_; }
      close(FTPH);

      my($siteadmin,$adminroot);
      foreach $siteadmin (@admins)
      {
        $adminroot = (getpwnam($siteadmin))[7];
        unless (-e "$adminroot/$FtpAdminHelp")
        {
 	  open(FHLP,"> $adminroot/.ftphelp");
	  print FHLP $ftphelpfile;
	  close(FHLP);
	  chown( (getpwnam($siteadmin))[2], (getgrnam($group))[2], "$adminroot/$FtpAdminHelp");
	  chmod(0664,"$adminroot/$FtpAdminHelp");
        }
      }
    }
  }
}

sub vsite_get_siteadmin_ftphelp
{
  my(@admins) = @_;
  
  # Remove site admin ftp instructions
  if ($Features{'proftpd'})
  {
    my($siteadmin,$adminroot);
    foreach $siteadmin (@admins)
    {
      $adminroot = (getpwnam($siteadmin))[7];
      unlink("$adminroot/$FtpAdminHelp") if (-e "$adminroot/$FtpAdminHelp");
    }
  }
}

1;
