# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Admserv.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Access_file = ".htaccess";
my $Adminuser = "admin";
my $Authtype = "Basic";
my $Htgroup = "/etc/htgroup";
my $Htgroup_dir = "/home/groups";
my $Htpasswd = "/etc/htpasswd";
my $Htusers_dir = "/home/users";
my $MainGroup = "home";
my $Sites_AdminGroup = "Site-adm";
my $Top_dir = "/usr/";
my $Usr_Authname = "CobaltServer";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Admserv.pm.  Changes made here will be lost.
package Cobalt::Admserv;

sub admserv_set_access
# admserv_set_access: set the access rights to a directory
# Arguments: path, mode, type, and name
#   mode may be "any", "admin", or "valid":
#    any -- open access
#    admin -- require user admin
#    site-admin -- require a site administrator
#    valid -- require valid-user
#    require -- require the given user
#    none -- no access restrictions (remove the file)
#   The type and name arguments is optional, unless the require mode is used
#   The admin user or group are always allowed access
# Return value: status string of the usual sort
# Side effects: creates the htaccess files
# Example: Cobalt::Admserv::admserv_set_access("/home/httpd/html/joe","require","user","joe");
{
    my ($dir,$mode,$type,$user) = @_;

    return "4001 Invalid directory $dir" 
      if (($dir !~ /^$Htusers_dir/) && ($dir !~ /^$Htgroup_dir/) && ($dir !~ /^$Top_dir/o));
    if ($mode eq "none")
    {
	if (-e "$dir/$Access_file")
	{
	    unlink("$dir/$Access_file") ||
		return "4002 Cannot remove $dir/.htaccess: $!";
	}
	return "2001 Opened access to $dir";
    }
    open(HTA,">$dir/$Access_file") ||
	return "4003 Cannot create $dir/.htaccess: $!";
    # No explicit <Limit>, because we want to restrict all methods
    print HTA "# Access file for $dir ($mode $type $user)\n";
    print HTA "order allow,deny\n";
    print HTA "allow from all\n";
    if    ($mode eq "valid")   { print HTA "require valid-user\n";   }
    elsif ($mode eq "admin")   { print HTA "require user $Adminuser\n";   }
    elsif ($mode eq "require") { print HTA "require $type $user $Adminuser\n";}
    elsif ($mode eq "site-admin") {
      print HTA "require group $MainGroup $Sites_AdminGroup $Adminuser\n";
    }
    print HTA "Authname $Usr_Authname\nAuthtype $Authtype\n";
    print HTA "AuthUserFile $Htpasswd\n";
    print HTA "AuthGroupFile $Htgroup\n";
    close HTA;

    # web server needs to read the file
    chmod 0644, "$dir/$Access_file";

    return "2002 Set access rights for $dir";
}


1;
