# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Ftp.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Badshell = "/bin/badsh";
my $FTPUsersMax = "10";
my $Ftpd_conf = "/etc/proftpd.conf";
my $Htgroup_dir = "/home/groups";
my $MainGroup = "home";
my $WWWuser = "httpd";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Ftp.pm.  Changes made here will be lost.
package Cobalt::Ftp;

sub Ftp_config_site
# Adds, Removes, or Modifies a virtual ftp site.
# Controls anonymous ftp access by group and ip (virtualhosts)
# Arguments: array of: group, ip, exists/doesn't, enable/disable anonymous ftp,
#                      quota for the incoming directory, maximum anonymous users
# Return value: string like "1104 Hotpants are not supported in this release"
# Example (Create/modify site with anonymous ftp): Cobalt::Ftp::Ftp_config_site("home","10.9.25.64",1,1);
# Example (Delete site): Cobalt::Ftp::Ftp_config_site("ispclient","10.9.25.64",0,1);
# Example (Disable anon ftp): Cobalt::Ftp::Ftp_config_site("ispclient","10.9.25.64",1,0);
{
	my($group,$ip,$exist,$anonftp,$quota,$maxusers) = @_;
	my $ftpuser = uc($group);
	$quota ||= 0; # No incoming dir if none specified

	return "4324 The default ftp site can not be disabled" if (($group eq $MainGroup) && !$exist);

	return "4325 Invalid maximum users: $maxusers" if ($maxusers =~ /\D/o);
	$maxusers = $FTPUsersMax unless ($maxusers =~ /^\d+$/o);

	my($ftpd_conf_file);
	
	return "4326 Invalid IP address: $ip" if (!Cobalt::Netutil::netutil_test_ipaddr($ip));
	return "4327 Invalid group name $group" if (!Cobalt::Group::group_exist($group));
	
	my(@sites) = Cobalt::Ftp::Ftp_list_site();
	# $create is set to 1 if the site is new; 0 if this is an existing
	# site.
	my($address,$create) = ('',1); # create flag for new, !create for modify
	# (the following line can replace the subsequent 5 lines)
	$create = 0 if (grep(/^$ip$/,@sites) || ($group eq $MainGroup));
	#foreach $address (@sites)
	#{
	#	$create = 0 if ($ip eq $address)
	#}
	#$create = 0 if ($group eq $MainGroup); # $MainGroup is always defined

	return "2047 Virtual FTP site was never defined" if (!$exist && $create);
	
	# This code is only used if we're enabling anonymous ftp
	# We build the entire block right here; we don't modify the file,
	# we just cut out the old block and paste in the new one.
	my($New_anonftp) = <<EOF;
        <Anonymous $Htgroup_dir/$group/ftp>
          User                          $ftpuser
          Group                         ftp
          UserAlias                     anonymous $ftpuser
          UserAlias                     guest $ftpuser
          MaxClients                    $maxusers

          <Directory *>
              <Limit WRITE>
              DenyAll
              </Limit>
          </Directory>

          <Directory $Htgroup_dir/$group/ftp/incoming>
	      Umask			002
              <Limit STOR>
              AllowAll
              </Limit>
              <Limit READ>
              DenyAll
              </Limit>
          </Directory>
        </Anonymous>
EOF

	if ($create)
	{
		# Make virtual host configuration changes here and below:
		my($site_additions) = <<EOF;
<VirtualHost $ip>
	Group			$group
	DefaultRoot		/ admin
	DefaultRoot		~/../.. Site-adm
	DefaultRoot		~ !Site-adm
	AllowOverwrite		on
	DisplayLogin		.ftphelp
EOF
		# Make the anonymoust FTP user and their directory
		# The FTP user shouldn't get a real shell
		mkdir("$Htgroup_dir/$group/ftp",02775);
		my $ret = Cobalt::User::user_add($ftpuser,
		     "$group anonymous FTP user","*",$quota,$Badshell,
		     0,"$Htgroup_dir/$group/ftp");
		     # return $ret if ($ret !~ /^2/o);
		chown( (getpwnam($WWWuser))[2], (getgrnam($group))[2], 
		       "$Htgroup_dir/$group/ftp");
		chmod(02775,"$Htgroup_dir/$group/ftp")
		  || warn "Can't chown $group/ftp: $!\n";
		mkdir("$Htgroup_dir/$group/ftp/incoming",02770);
		chown( (getpwnam($WWWuser))[2], (getgrnam($group))[2], 
		       "$Htgroup_dir/$group/ftp/incoming");
		chmod(02770,"$Htgroup_dir/$group/ftp/incoming");

		if ($quota) 
		{
			chmod(02773,"$Htgroup_dir/$group/ftp/incoming");
		}
		Cobalt::Admserv::admserv_set_access("$Htgroup_dir/$group/ftp/incoming/",
				   'require','group',"$group");

		# Add that big anonymous FTP block to the config file
		$site_additions .= $New_anonftp if ($anonftp);
		$site_additions .= "</VirtualHost>\n\n";

		# Cat to the end of the conf file
		open(FTPC,">> $Ftpd_conf") || return "4328 Cannot write ftp configuration file: $!";
		print FTPC $site_additions;
		close(FTPC);
		
		return "2048 Virtual FTP site created successfully";
	} else {
		# Enable/disable ftp, anonymous ftp, incoming 
		# Modify (not clobber) an existing record
		my($before_virtual_hosts_sections,$added_anon,$host_modified);

		Cobalt::Ftp::lock_ftp(1) || return "4329 FTP server in use, please try again soon";
		open(FTPD,"$Ftpd_conf")
		  || return "4330 Cannot read FTP config file /etc/proftpd.conf: $!";
		$before_virtual_hosts_sections = 1;
                while(<FTPD>)
                {
		    # The anonymous FTP block goes within the main body of the file...
		    # let's go all-out in having this be a special case
		    if ($before_virtual_hosts_sections && ($group eq $MainGroup))
		    {
			# We're in the main part of the config file, virtual hosts will
			# be at the end.
			if (/\<Anonymous\s+/ .. /\<\/Anonymous/)
			{
			    # the anonymous block
			    # we always ignore the anon block; either we wipe it, or
			    # we wipe it and replace it.

			    # Add on the new anonymous FTP settings for the default site
			    if ($exist && $anonftp && /\<\/Anonymous/)
			    {
				$ftpd_conf_file .= $New_anonftp."\n";
				$added_anon = 1;
			    }
			    next; # wiping out old anon block
			}
			elsif (/\<VirtualHost\s+/)
			{
			    # Default interface anonymous ftp goes in before any VirtualHost
			    $before_virtual_hosts_sections = 0;

			    if ($exist && $anonftp && !$added_anon)
			    {
				# Add on the new anonymous FTP settings for the default site
				$ftpd_conf_file .= $New_anonftp."\n";
				$added_anon = 1;
			    }
			}

			# we're not in anon block...so we just leave it be.
			$ftpd_conf_file .= $_;
		    }
		    # Now that I know about this .. operator, I'm going to use it
		    # ALL THE TIME.
		    elsif ((/\<VirtualHost\s+$ip\>$/) .. (/\<\/VirtualHost/))
		    {
			# we are in the VirtualHost block for the site we care about
			$host_modified = 1;
			$before_virtual_hosts_sections = 0;

			# Is this site is supposed to exist?
			next unless ($exist);

			# We wipe any lines within the anonymous block for this site, and then
			# re-add the configuration directives from scratch.
			if (/\<Anonymous\s+$Htgroup_dir\/($group)\// .. /\<\/Anonymous/)
			{
			    $ftpd_conf_file .= $New_anonftp."\n"
				if ($anonftp && /\<\/Anonymous/);
			    next;
			}

			# we're not in anon block...so we just leave it be.
			$ftpd_conf_file .= $_;
		    }
		    else
		    {
			# another one of these anal (uhh, anon) blocks..
			# wipe it out if updating maingroup...it will be added later
			next if (($group eq $MainGroup) && (/\<Anonymous\s+$Htgroup_dir\/$MainGroup\/ftp/o .. /\<\/Anonymous/));

			# We're in some other block we're not messing with
			$ftpd_conf_file .= $_;
		    }
		}
                close(FTPD);

		# Add default anonymous ftp service if no VirtualHosts are present
		$ftpd_conf_file .= $New_anonftp."\n" if ($exist && $anonftp && !$added_anon && ($group eq $MainGroup));

		return "4331 FTP virtual host $ip not found" if (!$host_modified && ($group ne $MainGroup));

		open(FTPD,"> $Ftpd_conf") || return "4332 Could not edit FTP configuration";
		print FTPD $ftpd_conf_file;
		close(FTPD);
		Cobalt::Ftp::lock_ftp(0);
		
		mkdir("$Htgroup_dir/$group/ftp",0755) unless (-d "$Htgroup_dir/$group/ftp");
		
		if ($quota && $anonftp) {
			chmod(02773,"$Htgroup_dir/$group/ftp/incoming");
		} else {
			chmod(02770,"$Htgroup_dir/$group/ftp/incoming");
		}
		if ($quota) {
                        my($ret) = Cobalt::Ftp::Ftp_set_incoming_quota($group,$quota);
                        return $ret if ($ret !~ /^2\d\d\d\s/o);
                }

		return "2049 Virtual FTP site modified successfully" if ($exist);

		# We're deleting an ftp site, take out the anonymous user too
		my($ret) = Cobalt::User::user_del($ftpuser,1);
		return $ret if ($ret !~ /^2/o);

		return "2050 Virtual FTP site removed successfully";
	}
}


1;
