# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Group.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Adminuser = "admin";
my $Firstgid = "110";
my $Group_skel = "/etc/skel/group";
my $Gtmp = "/etc/locks/gtmp";
my $Htgroup_dir = "/home/groups";
my $Htgtmp = "/etc/locks/htgtmp";
my $Htscript = "/etc/rc.d/init.d/httpd.init";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $MainGroup = "home";
my $MaxMembers = "60000";
my $MaxNameLen = "12";
my $MaxQuota = "10240000";
my $Private_html = "private";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Subsys = "/var/lock/subsys";
my $WWWuid = "15";
my $WWWuser = "httpd";
my %Features = ("networker","1","telnetConfig","0","dhcpd","1","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","1","www","1","mx","1","gated","1","dns","1","nfs","0","english","1","atalk","1","fpx","1","siteAdminRedirect","0","users","1","ntp","0","search","1","anonFTP","1","specialHomeSkel","0","viewGroups","1","cobaltHomePage","1","lastNameFirst","1","vsites","0","discuss","1","docorg","0","groups","1","nas","0","smb","1","mail","1","autoGenWebUsage","1","squid","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","0","mon_port_testing","1","viewUsers","1","usersCGIInWizard","1","privatedirs","1","adm","1","siteManagement","0","ssl","0","personalRedirect","0","webRequests","1","usersAddUser","1","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my %System_groups = ("tty","1","mail","1","httpd","1","root","1","gopher","1","squid","1","admin","1","ftp","1","kmem","1","nobody","1","disk","1","games","1","global","1","users","1","adm","1","Site-adm","1","daemon","1","wheel","1","bin","1","homes","1","lp","1","man","1","news","1","uucp","1","dip","1","mem","1","sys","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Group.pm.  Changes made here will be lost.
package Cobalt::Group;

sub group_add
# group_add: Create a group
# Arguments: the name of the group, group quota, and (optional) list of members
# Side effects: adds entries to the group file
# Return value: the usual status string of format "816 I said NO camels"
{
    my ($return,$gid);
    return "1253 Groups not enabled" unless ($Features{'groups'});
    my ($group,$quota,@members) = @_;
    return "4234 Invalid group name $group" if ($group !~ /^[\w\-\.]+$/o);
    return "4242 Group names must be no longer than 12 characters."
	if (length($group) > $MaxNameLen);
    my $warn_quota = ($MaxQuota / 1024);
    return "4243 Quotas may not exceed $warn_quota"
	if ($quota > $MaxQuota);
    return "4267 The name $group is reserved by the system" 
	if ($System_groups{$group});
    return "4299 Group $group already exists"
	if (getgrnam($group));
    return "4240 Name $group already in use as a user" 
	if (getpwnam($group));
    return "4241 Name $group is already in use as a mailing list or alias"
	if (Cobalt::List::alias_exist($group));
    return 
	Cobalt::Group::group_abort("4244 Too many group members; you may only have 60000 members per group")
	    if ($#members >= $MaxMembers);

    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    ### Add to the files in /etc
    setgrent();
    for ($gid = $Firstgid; getgrgid($gid); $gid++) {;}
    my $line = "${group}::${gid}:" . join(',',@members,$WWWuser,$Adminuser) . "\n";
    my $htline = "${group}:" . join(' ',@members,$Adminuser). "\n";
    print STDERR "Adding to $Gtmp: $line" if ($Debug);
    open(GTMP,">>$Gtmp") 
	|| return Cobalt::Group::group_abort("4222 Cannot append to group lock file /etc/locks/gtmp: $!");
    print GTMP $line;
    close GTMP;
    open(GTMP,">>$Htgtmp") 
	|| return Cobalt::Group::group_abort("4223 Cannot append to /etc/locks/htgtmp: $!");
    print GTMP $htline;
    close GTMP;

    ### Make the directory
    if ($Features{'groupdirs'}) {
      my $home = "$Htgroup_dir/$group";
      print STDERR "Creating directories under $home\n" if ($Debug);
      return Cobalt::Group::group_abort("4226 Group home $home already exists") if (-e $home);
      
      # Make sure the top-level group directory exists
      unless (-d $Htgroup_dir)	{
	# this line doesn't chmod sometimes
	mkdir($Htgroup_dir,0755);
	chmod 0755, $Htgroup_dir;

	if ($Features{'atalk'}) {
	  # AppleTalk seems to feel some odd need for this directory.
	  mkdir("$Htgroup_dir/Network Trash Folder",01777)
	    || return Cobalt::Group::group_abort("4247 Cannot make /home/groups/Network Trash Folder: $!");
	  chmod(01777,"$Htgroup_dir/Network Trash Folder");
	}
      }
      
      mkdir($home,02775) 
	|| return Cobalt::Group::group_abort("4227 Cannot create home $home: $!");
      chown($WWWuid,$gid,$home);
      # The setgid bit means that all files or subdirectories will be created
      # also owned by this group.  The user writable bit has to be set or
      # appletalk won't work
      chmod(02775,$home);
      
      if ($Features{'privatedirs'})	{
	mkdir("$home/$Private_html",02770) 
	  || return Cobalt::Group::group_abort("4228 Cannot create group private area $home/private: $!");
	chown($WWWuid,$gid,"$home/$Private_html");
	chmod(02770,"$home/$Private_html");
      }
      
      # Copy *.html from the skel directory into the new group home area
      opendir(SKEL,$Group_skel)
    	|| return Cobalt::Group::group_abort("3228 Cannot read /etc/skel/group: $!");
      my ($htmlfile);
      foreach $htmlfile (grep(/\.html$/o,readdir(SKEL)))	{
	next if ($htmlfile eq "index.html"); # special case, do later
	print STDERR "Copying $htmlfile to $home/$htmlfile\n"
	  if ($Debug);
	
	# inline the cp subroutine for speed
	open(SRC,"$Group_skel/$htmlfile") 
	  || return Cobalt::Group::group_abort("4224 Cannot open source file /etc/skel/group/$htmlfile: $!");
	open(DEST,">$home/$htmlfile") 
	  || return Cobalt::Group::group_abort("4225 Cannot open destination file $home/$htmlfile: $!");
	while (<SRC>)
	  {
	    # No substitutions
	    print DEST;
	  }
	close DEST;
	close SRC;
	
	chown($WWWuid,$gid,"$home/$htmlfile");
	chmod(0664,"$home/$htmlfile");
      }
      closedir SKEL;

      # now do index.html; this is a special case because we do some macro
      # substitution on it, e.g. for the group name
      my $cgroup = ucfirst($group);
      
      my $index_skel = "$Group_skel/index.html";
      if ($Features{'specialHomeSkel'})	{
	$index_skel = "$Group_skel/$MainGroup/index.html" 
	  if ($group eq $MainGroup);
      }
      
      open(SRC,$index_skel) 
	|| return Cobalt::Group::group_abort("4277 Cannot open source file /etc/skel/group/index.html: $!");
      open(DEST,">$home/index.html") 
	|| return Cobalt::Group::group_abort("4278 Cannot open destination file $home/index.html: $!");
      while (<SRC>) {
	if (/\[/o)	  {
	  s/\[GROUP\]/$cgroup/;
          s/\[RGROUP\]/$group/;
	  s/\[PATH\]/$home\/index.html/;
	}
	print DEST;
      }
      close DEST;
      close SRC;
      
      chown($WWWuid,$gid,"$home/index.html");
      chmod(0664,"$home/index.html");
      
      if ($Features{'privatedirs'}) {
	# lock down access to the private directory
	Cobalt::Admserv::admserv_set_access("$home/$Private_html","require","group",$group);
	
	# now do index.html for the private area
	if (-e "$Group_skel/private.tmpl") {
	  my $cgroup = ucfirst($group);
	  open(SRC,"$Group_skel/private.tmpl") 
	    || return Cobalt::Group::group_abort("4279 Cannot open source file /etc/skel/group/private.tmpl: $!");
	  open(DEST,">$home/$Private_html/index.html") 
	    || return Cobalt::Group::group_abort("4280 Cannot open destination file $home/private/index.html: $!");
	  while (<SRC>) {
	    if (/\[/o) {
	      s/\[GROUP\]/$cgroup/;
	      s/\[PATH\]/$home\/$Private_html\/index.html/;
	    }
	    print DEST;
	  }
	  close DEST;
	  close SRC;
	  
	  chown($WWWuid,$gid,"$home/$Private_html/index.html");
	  chmod(0664,"$home/$Private_html/index.html");
	}
      }

      Cobalt::Group::samba_add($home,$group) if $Features{'smb'};
      
      my $atalk_ret = Cobalt::Atalk::atalk_set_volume($home, $group) if $Features{'atalk'};
      return Cobalt::Group::group_abort($atalk_ret) if ($atalk_ret !~ /^2/o);

    } # if groupdirs

    # not useful for new discussion group
#    Cobalt::Group::group_mkdiscuss($group,$gid) if ($Features{'discuss'});

    Cobalt::List::list_create($group,@members);  # no checkee returnee

    ### end game
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    Cobalt::Group::group_set_quota($group,$quota);

    print STDERR "Completed adding $group\n" if ($Debug);

    # We restart the web server so it re-reads its group membership;
    # otherwise it may not be able to read group files properly.
    # If we just hup'ed, there would be no permissions set
    # For a virtual machine model a group is only added 
    #   with a virtual interface, which then restarts
    #   the web server, no need to restart twice
    unless($Features{'vmachine'})
    {
        # Restart the web server, if it's up
        if (-f "$Subsys/httpd")
        {
            system("$Htscript stop >>$Sauce_log 2>&1");
            system("$Htscript start >>$Sauce_log 2>&1");
            system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
        }
    }
    return "2237 Group $group created successfully";
}


1;
