# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Group.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Adminuser = "admin";
my $Gtmp = "/etc/locks/gtmp";
my $Htgtmp = "/etc/locks/htgtmp";
my $Lockdir = "/etc/locks";
my $MaxMembers = "60000";
my $WWWuser = "httpd";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Group.pm.  Changes made here will be lost.
package Cobalt::Group;

sub group_set_membership
# group_set_membership: set the members list for a group
# Arguments: the name of the group and an array of members
# Side effects: modifies /etc/group, /etc/htgroup
#   This function ensures that the list contains no duplicate names.
{
    my ($group,@list) = @_;
    return Cobalt::Group::group_abort("4210 No such group: $group") unless (getgrnam($group));
    return 
	Cobalt::Group::group_abort("4245 Too many members; you may only have 60000 members per group")
	    if ($#list >= $MaxMembers);

    print STDERR "Membership of $group is being set to @list\n" if ($Debug);

    # Using the hash table is a cute way to guarantee no repeat
    # entries
    my %temp = map { $_, 1} @list;
    $temp{$WWWuser} = 1;   # guarantee membership for the web server user
    $temp{$Adminuser} = 1; # ditto for admin
    my $list = join(',',sort(keys(%temp)));

    # Don't go to any trouble if nothing is changing
    my $oldmembers = join(',',Cobalt::Group::group_show($group));
    if ($oldmembers eq $list)
    {
	return "2224 No change to group membership";
    }

    # fix the group file
    open(GROUP,$Gtmp) ||  return Cobalt::Group::group_abort("4211 Cannot read /etc/locks/gtmp: $!");
    open(GTMP,">$Lockdir/group") 
	|| return Cobalt::Group::group_abort("4212 Cannot append to /etc/locks/group: $!");
    "$group:x:" =~ /^$group:(.+):/;   # pre-compiled regexps are better
    while (<GROUP>)
    {
	if (//)
	{
	    print GTMP "$group:$1:$list\n";
	}
	else
	{
	    print GTMP;
	}
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/group",$Gtmp) ||
	return Cobalt::Group::group_abort("4213 Cannot replace /etc/locks/gtmp: $!");

    # Fix the htgroup file
    $list = join(' ',keys(%temp));
    open(GROUP,$Htgtmp) ||  return Cobalt::Group::group_abort("4214 Cannot read /etc/locks/htgtmp: $!");
    open(GTMP,">$Lockdir/htgroup") 
	|| return Cobalt::Group::group_abort("4215 Cannot append to /etc/locks/htgroup: $!");
    "$group:" =~ /^$group:/;   # pre-compile for speed
    while (<GROUP>)
    {
	if (//)
	{
	    print GTMP "$group:$list\n";
	}
	else
	{
	    print GTMP;
	}
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/htgroup",$Htgtmp) ||
	return Cobalt::Group::group_abort("4216 Cannot replace /etc/locks/htgtmp: $!");

    # Fix the list membership to match the group membership
    Cobalt::List::list_set_mem($group,@list);
    return "2225 Group membership set successfully";
}


1;
