# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Nat.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Filter_link = "/etc/rc.d/rc3.d/S11ipfilters";
my $Filter_script = "/etc/rc.d/init.d/ipfilters";
my $Lockdir = "/etc/locks";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Nat.pm.  Changes made here will be lost.
package Cobalt::Nat;

sub nat_enable
# Enable or disable the filtering rules both now and at boot
# Arguments: 0 = disable, 1 = enable filtering, 2 = enable
#   masquerading, 3 = enable both filtering and masquerading
# Side effects: runs the ip filter script, sets up the boot-time symlink,
#   modifies $MODE in the script
# Return value: status string
# Note: it's a bad idea to have the rules go into effect immediately while
#   the user is editing them.  If they do their edits in the wrong order,
#   things could get really weird and connections could die.
{
  my ($filter_mode) = @_;

  # Set the $MODE variable in the filter script
  return "4713 You must create filtering/masquerading rules before you can enable them" if (! -e $Filter_script);
  return "4714 The filtering rules are being edited.  Try again later." if (-e "$Lockdir/ipfilters");
  open(LOCK,">$Lockdir/ipfilters") || return "4715 Cannot write new filtering rules: $!";
  open(NAT,$Filter_script) || return "4716 Cannot read default IP filter configuration: $!";
  while (<NAT>) {
    if (/^MODE=(\d+)/o) {
      print LOCK "MODE=$filter_mode\n";
    } else {
      print LOCK;
    }
  }
  close NAT;
  close LOCK;
  rename("$Lockdir/ipfilters",$Filter_script)
    || return "4717 Cannot write IP filter rules: $!";
  chmod(0755,$Filter_script);

  # Set the symlink.  The symlink should be there regardless,
  # because we turn things on and off with the $MODE variable
  # at the top of the ipfilters script.
  # We know the filter script exists, because we already exited
  # above if it doesn't.
  if( ! -l $Filter_link ) {
    symlink($Filter_script,$Filter_link)
      || return "4718 Cannot set IP filter boot script: $!";
  }

  # Now put those rules (or lack of rules) into effect
  system("$Filter_script >>$Sauce_log 2>&1")
    && return "4719 Cannot run IP filter script: $!";

  # Give a return code so we know what just happened...
  if ($filter_mode) {
	return "2092 Filters enabled";
  } else {
	return "2093 Disabled IP filtering";
  }
}


1;
