# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from User.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Htpasswd = "/etc/htpasswd";
my $Htptmp = "/etc/locks/htptmp";
my $Lockdir = "/etc/locks";
my $Passwd = "/etc/passwd";
my $Ptmp = "/etc/ptmp";
my $Smbpasswd = "/etc/smbpasswd";
my $Smbptmp = "/etc/locks/smbptmp";
my %Features = ("networker","1","telnetConfig","0","dhcpd","1","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","1","www","1","mx","1","gated","1","dns","1","nfs","0","english","1","atalk","1","fpx","1","siteAdminRedirect","0","users","1","ntp","0","search","1","anonFTP","1","specialHomeSkel","0","viewGroups","1","cobaltHomePage","1","lastNameFirst","1","vsites","0","discuss","1","docorg","0","groups","1","nas","0","smb","1","mail","1","autoGenWebUsage","1","squid","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","0","mon_port_testing","1","viewUsers","1","usersCGIInWizard","1","privatedirs","1","adm","1","siteManagement","0","ssl","0","personalRedirect","0","webRequests","1","usersAddUser","1","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/User.pm.  Changes made here will be lost.
package Cobalt::User;

##############################

sub lock_passwd
# Lock the password file so that nobody can collide with us.
# Arguments: none
# Side effects: copies $Passwd to $Ptmp and $Htpasswd to $Htptmp
# Return value: message of the form "2671 All of the engineers prefer MIPS"
{
    if ((-e $Ptmp) || (-e $Htptmp))
    {
      return Cobalt::User::user_abort("4507 The password file is locked.");
    }
    else
    {
	# Lock the password file by creating the ptmp file as a
	# copy of the password file.
	open(PASSWD,$Passwd) || return "4508 Cannot lock /etc/passwd: $!";
	open(PTMP,">$Ptmp") 
	    || return "4509 Cannot lock /etc/ptmp: $!";
	print PTMP while (<PASSWD>);
	close PTMP;
	close PASSWD;

	# Lock the smbpasswd file by creating the smbptmp file as a
	# copy of the password file.
	if ($Features{'smb'}) {
	  open(SMBPASSWD,$Smbpasswd) || return "4572 Cannot lock /etc/smbpasswd: $!";
	  open(SMBPTMP,">$Smbptmp") 
	    || return "4573 Cannot lock /etc/locks/smbptmp: $!";
	  print SMBPTMP while (<SMBPASSWD>);
	  close SMBPTMP;
	  close SMBPASSWD;
	}

	# Lock the netatalk passwd file by calling atalk_lock_passwd
	# (which does the same thing as the others above)
	if ($Features{'atalk'}) {
	  my $atalk_ret = Cobalt::Atalk::atalk_lock_passwd();
	  return $atalk_ret if ($atalk_ret !~ /^2/);
	}

	# we need to lock this now in case the next opens abort
	open(LOCK,">$Lockdir/user-$$") || return Cobalt::User::user_abort("4599 Cannot get user lock: $!");
	print LOCK "$$\n$0";
	close LOCK;

	print STDERR "Copying htpassword file $Htpasswd to $Htptmp\n" 
	    if ($Debug);
	if (! -e $Htpasswd)
	{
	    # this looks like a fresh system
	    print STDERR "Creating new htpassword file $Htpasswd\n" 
		if ($Debug);
	    open(HTPASS,">$Htpasswd")
	      || return "4574 Cannot create /etc/htpasswd: $!";
	    close HTPASS;
	}
	Cobalt::Utility::sauce_cp($Htpasswd,$Htptmp);
    }

    print STDERR "Locked $Ptmp and $Htptmp\n" if ($Debug);
    return "2511 Password file locked";
}


1;
