# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from User.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Adminuser = "admin";
my $Htpasswd = "/etc/htpasswd";
my $Htptmp = "/etc/locks/htptmp";
my $Lockdir = "/etc/locks";
my $Mailspool = "/home/spool/mail";
my $Ptmp = "/etc/ptmp";
my $Smbptmp = "/etc/locks/smbptmp";
my %Features = ("networker","1","telnetConfig","0","dhcpd","1","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","1","www","1","mx","1","gated","1","dns","1","nfs","0","english","1","atalk","1","fpx","1","siteAdminRedirect","0","users","1","ntp","0","search","1","anonFTP","1","specialHomeSkel","0","viewGroups","1","cobaltHomePage","1","lastNameFirst","1","vsites","0","discuss","1","docorg","0","groups","1","nas","0","smb","1","mail","1","autoGenWebUsage","1","squid","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","0","mon_port_testing","1","viewUsers","1","usersCGIInWizard","1","privatedirs","1","adm","1","siteManagement","0","ssl","0","personalRedirect","0","webRequests","1","usersAddUser","1","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my %System_users = ("mail","1","httpd","1","root","1","gopher","1","squid","1","guest","1","anonymous","1","halt","1","ftp","1","nobody","1","operator","1","games","1","adm","1","daemon","1","bin","1","shutdown","1","lp","1","sync","1","news","1","uucp","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/User.pm.  Changes made here will be lost.
package Cobalt::User;

sub user_del
# user_del: Delete a user
# Arguments: the login name of the user, optional boolean value to not delete home directory
# Side effects: removes the password file entry, and deletes the home
#   directory
# Return value: a string of the form "902 nut crunchies exploded"
# Example: Wipes user and home directory
#          Cobalt::User::user_del("joey"); 
# Example: Deletes user but leaves the user home directory as-is
#          Cobalt::User::user_del("anonftpuser",1); 
# Example: Deletes user but leaves the user home directory as-is
#          Cobalt::User::user_del("anonftpuser","/home/sites/home/ftp/");
{
    my ($username,$userhome) = @_;
    my ($return);
    my ($login,$home) = (getpwnam($username))[0,7];
    return "2516 Cannot delete $username: no such user"
	unless ($login);
    return "4502 Cannot delete admin" if ($username eq $Adminuser);
    return "4522 Cannot delete $username: prohibited user"
	if ($System_users{$username});
    
    return $return unless (($return = Cobalt::User::lock_passwd()) =~ /^2/o);

    #   clean the smbpasswd file
    if ($Features{'smb'}) {
      print STDERR "Cleaning from $Smbptmp\n" if ($Debug);
      open(SMBPTMP,$Smbptmp) 
	|| return Cobalt::User::user_abort("4576 Cannot open /etc/locks/smbptmp to delete user: $!");
      open(SMBPASSWDLOCK,">$Lockdir/smbpasswd") 
	|| return Cobalt::User::user_abort("4577 Cannot clobber /etc/locks/smbpasswd: $!");
      while (<SMBPTMP>)
	{
	  print SMBPASSWDLOCK unless (/^$username/o);
	}
      close SMBPASSWDLOCK;
      close SMBPTMP;
      rename("$Lockdir/smbpasswd",$Smbptmp) 
	|| return Cobalt::User::user_abort("4578 Cannot move /etc/locks/smbptmp: $!");
    }

    # clean the netatalk passwd file
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_del_user($username);
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    ### clean the password file
    print STDERR "Cleaning from $Ptmp\n" if ($Debug);
    open(PASSWD,$Ptmp) 
	|| return Cobalt::User::user_abort("4517 Cannot open /etc/ptmp to delete user: $!");
    open(PTMP,">$Lockdir/passwd") 
	|| return Cobalt::User::user_abort("4518 Cannot clobber /etc/locks/passwd: $!");
    "$username:" =~ /^$username:/; # pre-compile our pattern
    while (<PASSWD>)
    {
	print PTMP unless (//);
    }
    close PTMP;
    close PASSWD;
    rename("$Lockdir/passwd",$Ptmp) 
	|| return Cobalt::User::user_abort("4519 Cannot move /etc/ptmp: $!");
    
    # Clean the htpasswd file
    open(HTPASSWD,$Htptmp) 
	|| return Cobalt::User::user_abort("4520 Cannot open htpasswd file /etc/htpasswd: $!");
    open(HTP,">$Lockdir/htpasswd") 
	|| return Cobalt::User::user_abort("4521 Cannot clobber /etc/locks/htptmp: $!");
    "$username:" =~ /^$username:/; # pre-compile
    while (<HTPASSWD>)
    {
	print HTP unless (//);
    }
    close HTP;
    close HTPASSWD;
    rename("$Lockdir/htpasswd",$Htptmp) 
	|| return Cobalt::User::user_abort("4540 Cannot rename /etc/locks/htptmp: $!");
    
    ### get rid of the directory
    unless ($userhome) 
    {
        return Cobalt::User::user_abort("4531 $username has no files.") 
	    unless ($home && (-d $home));
        print STDERR "Deleting $home...\n" if ($Debug);
        Cobalt::Utility::sauce_deltree($home);
    } else
    {
        print STDERR "Deleting user but leaving directories $userhome alone...\n" if ($Debug);
    }
    unlink("$Mailspool/$username") if (-e "$Mailspool/$username");

    ### Commit
    return $return unless (($return = Cobalt::User::commit_passwd()) =~ /^2/o);
    
    ### Clean the group file
    my ($gr,$gp,$gd,$members,@ingroups);
    setgrent();
    $username =~ /\b$username\b/;  # pre-compile
    # We have to do this in two steps because the getgrent
    # stepping through the group list gets confused when we
    # do the getgrnam call within Cobalt::Group::group_del_user()
    while (($gr,$gp,$gd,$members) = getgrent)
    {
	push(@ingroups,$gr)
	    if ($members =~ //);
    }
    endgrent();
    foreach $gr (@ingroups)
    {
	Cobalt::Group::group_del_user($gr,$username);
    }

    # Make sure that we don't leave any vacation or forwarding stuff
    # around
    # I'm not going to sweat the return values here
    Cobalt::Vacation::vacation_set_message($username,"")
	if (Cobalt::Vacation::vacation_get_message($username));
    Cobalt::Vacation::vacation_set_on($username,0) if (Cobalt::Vacation::vacation_get_on($username));
    Cobalt::Vacation::vacation_del_db($username);

    if ($Features{'vmachine'}) {
      # Remove the user from all mailing lists (including site-users) here
      # rationale: list-admin does not want to process bounce mails
      # problem: where do i find the domainname user is bound to? w/o
      #          that user@domain entries cannot be safely deleted
      # fix: use list_sites_del_mem to find the users domain!!
      my $domain = Cobalt::List::list_sites_del_mem($username);
      Cobalt::List::list_lists_del_mem("$username\@$domain");
    }
    # delete unqualified:-) user entries from all the mailing lists
    Cobalt::List::list_lists_del_mem($username);

    # Make sure email aliases are removed
    Cobalt::Email::mail_virtuser_del_byuser( $username );
    my $alias;
    foreach $alias (Cobalt::List::alias_get_bytarget($username)) {
      Cobalt::List::alias_delete($alias);
    }
    
    
    print STDERR "Deletion of user $username complete\n" if ($Debug);
    return "2535 User $username deleted";
}


1;
